/*****
 *
 * TBR61.PRG
 *
 * Luiz F. Quintela
 * Copyright (c) 1991 Nantucket Corporation.
 *                    All Rights Reserved.
 *
 * Clipper tbr61 /N /W
 * RTLINK FILE tbr61
 *
 */

#include "inkey.ch"
#include "setcurs.ch"

#define     COLSEP     CHR(32)  + CHR(179) + CHR(32) 
#define     HEADSEP    CHR(205) + CHR(209) + CHR(205)
#define     FOOTSEP    CHR(205) + CHR(207) + CHR(205)
#define     MYCOLORS   "W+/BG,W+/B"

#xcommand      STABILIZE <obj> INKEY [TO] <x>  =>;
               DISPBEGIN();;
               WHILE (!<obj>:stabilize()) .AND.;
                    ((<x> := INKEY()) == 0);;
               END;;
               DISPEND()

#define   MAXLEN    14


FUNCTION Main()
   LOCAL b, nKey, column, bBlock, nGTotal
   LOCAL nSubscript := 1
   LOCAL k := 0
   LOCAL aArray := { 100.34, 45.90, 123.00, -6.89,;
                      12.98,  3.05, 945.00, -4.01,;
                     145.89, 55.78, -34.00, 39.33,;
                      16.00, 18.34;
                   }

   SET(_SET_SCOREBOARD, .F.)
   SETCURSOR(SC_NONE)
   SETCOLOR("N/W")
   SCROLL()
   @  2,14 TO 21,36 DOUBLE COLOR MYCOLORS
   @ MAXROW(), 0 SAY PADR("ESC - Quit" +;
                          SPACE(20) +;
                          "Enter - EDIT",;
                          MAXCOL() + 1) ;
                          COLOR "W+/RB"

   // TBrowse object for values
   b := TBrowseNew( 3, 15, 20, 35 )
   b:colorSpec := MYCOLORS
   b:headSep   := HEADSEP
   b:colSep    := COLSEP
   b:footSep   := FOOTSEP
   b:goTopBlock    := {|| nSubscript := 1}
   b:goBottomBlock := {|| nSubscript := MAXLEN}
   b:skipBlock := {|x| ;
                 k := IF(ABS(x) >= IF(x >= 0,;
                 MAXLEN - nSubscript, nSubscript - 1),;
                 IF(x >= 0, MAXLEN - nSubscript,;
                 1 - nSubscript),;
                 x), nSubscript += k,;
                 k }

   // Total
   bBlock := {|| nGTotal := 0,;
                 AEVAL(aArray, {|x| nGTotal += x}),;
                 "Total: " + STR(nGTotal,12,2)}

   // TBColumn Object
   column := TBColumnNew("Values",;
               {|p| IF(p == NIL, aArray[nSubscript],;
                    aArray[nSubscript] := p)})
   column:width   := 20 // NOTE
   column:footing := EVAL(bBlock)
   b:addColumn(column)

   WHILE .T.
      STABILIZE b INKEY TO nKey

      IF (b:stable)
         nKey := INKEY(0)

      ENDIF

      IF !TbMoveCursor( nKey, b )
         IF (nKey == K_ESC)
            EXIT

         ELSEIF (nKey == K_ENTER)
            DoGet( b )
            //
            // This is not a generic solution
            // since "column" assumes you are
            // already in the proper place
            // or you have only one column
            //
            // A generic solution should use
            // browse:getColumn(browse:colPos)
            // in order to get the reference 
            // to the current column
            //
            column:footing := EVAL(bBlock)
            b:configure()
            b:invalidate()

         ENDIF

      ENDIF

   END
   SCROLL()
   RETURN (NIL)

/*****
 *
 * Cursor Movement Methods
 *
 */

STATIC FUNCTION TBMoveCursor( nKey, oObj )
   LOCAL nFound
   STATIC aKeys := ;
       { K_DOWN      , {|b| b:down()},;
         K_UP        , {|b| b:up()},;
         K_PGDN      , {|b| b:pageDown()},;
         K_PGUP      , {|b| b:pageUp()},;
         K_CTRL_PGUP , {|b| b:goTop()},;
         K_CTRL_PGDN , {|b| b:goBottom()},;
         K_RIGHT     , {|b| b:right()},;
         K_LEFT      , {|b| b:left()},;
         K_HOME      , {|b| b:home()},;
         K_END       , {|b| b:end()},;
         K_CTRL_LEFT , {|b| b:panLeft()},;
         K_CTRL_RIGHT, {|b| b:panRight()},;
         K_CTRL_HOME , {|b| b:panHome()},;
         K_CTRL_END  , {|b| b:panEnd()} }

   nFound := ASCAN( aKeys, nKey )
   IF (nFound != 0)
      EVAL( aKeys[++nFound], oObj )

   ENDIF
   RETURN (nFound != 0)

/*****
 *
 * @...GET
 *
 */

STATIC FUNCTION DoGet( obj )
   LOCAL nCursSave
   LOCAL column, get, nKey

   nCursSave := SETCURSOR(SC_NORMAL)
   WHILE ( !obj:stabilize() )
   END
   column := obj:getColumn( obj:colPos )
   get := GetNew(ROW(), COL(), column:block,;
               column:heading,, obj:colorSpec)
   READMODAL( {get} )
   obj:refreshCurrent()
   nKey := LASTKEY()
   IF ( nKey == K_UP .OR. nKey == K_DOWN .OR. ;
      nKey == K_PGUP .OR. nKey == K_PGDN )
      KEYBOARD CHR( nKey )

   ENDIF
   SETCURSOR(SC_NONE)
   RETURN (NIL)

// EOF - TBR61.PRG //

