******************************************************************************
*
*
* FCO Compiler Beispielprogramm
*
* Das ist ein Beispiel fr ein TSR-Programm zur Demonstartion der TSR-
* mglichkeiten in FCO. Das Programm verwaltet Namen und Telefonnummern.
*
* Einige Informationen ber spezielle Funktionen:
*
* PROC_ENTRY()
*     Diese Prozedur initialisiert den Einsprungpunkt fr das TSR-
* programm. Ein TSR-Programm kann durchaus mehrere Einsprungpunkte haben,
* die jeweils ber eine andere "HotKey"-Kombination angespochen werden.
* Die formalen Parameter von PROC_ENTRY() lauten:
*
*         PROC_ENTRY( <Prozedurname>, <"Hot Keys"> )
*
* PROC_ENTRY gibt einen logischen Wert zurck. Ist er .T., wurde die an-
* gegebene Prozedur in die "HotKey"-Liste bernommen, bei .F. wurde eine
* unzulssige Tastensequenz angegeben. Danach mu das Programm zustzlich
* in den residenten Zustand versetzt werden.
*
* Gltige "HorKey"-Angaben sind:
*
* <Alt>, <Ctrl>, <Leftshift>, <Rightshift>, <Shift>, <A...Z>, <F1..F10>
*
* ein Beispiel:
*
*     IF proc_entry( MeineProc, "<Ctrl><Alt><F10>" )
*           ...
*
* "MeineProc" wir immer aufgerufen, wenn die Tasten "Ctrl", "Alt" und "F10"
* gleichzeitig gedrckt werden.
*
* Um ein Programm zu beenden uns resident zu halten (Terminate and Stay
* Resident), verwenden Sie die TSR()-Funktion. Mit dem angegebenen Parmameter
* legen Sie die Gre des vom TSR benutzten Speichers fest. FCO belegt
* mindestens 16KByte als Puffer fr Datei-Operationen. Je grer Sie den
* Speicher festlegen, um so schneller luft spter das Programm (allerdings
* nehmen Sie zustzlich freien Speicher weg, der anderen Anwendungen fehlen
* knnte).
*
* Beispiel:
*
* IF .NOT. proc_entry( entry, "<Alt><Leftshift>" )
*     ? "Kann Programm nicht resident laden"
* ELSE
*     TSR( 0 )                  && kleinstmglichen Speicher verwenden
* ENDIF
*
*
******************************************************************************

#include tsr.hdr
#include data.hdr
#include colors.hdr
#include io.hdr
#include database.hdr
#include string.hdr
#include system.hdr
#include keys.hdr
#include error.hdr
#include warn.hdr

*******************************************************************************
*
*  Definitionen
*
*******************************************************************************

#define     SINGLE_BOX "Ŀ"
#define     DOUBLE_BOX "ͻȺ"

#define     M_SEARCH 0
#define     M_NEXT   1
#define     M_ADD  2
#define     M_EDIT    3
#define     M_DELETE 4

#define     CURLENGTH 50

*******************************************************************************
*
*  Datenbank-Struktur
*
*******************************************************************************

DBFDEF phone
   CHAR(23)    name
   CHAR(23)    phone
ENDDEF

*******************************************************************************
*
*  Globale Variablen
*
*******************************************************************************

VARDEF
    CHAR(80)    phone_name
    CHAR(30)    srch_name, srch_phone
    INT         cur_row      = 9
    INT         top_row      = 9
    INT         top_col      = 15
    INT         bot_row      = 20
    INT         bot_col      = 64
    INT         fcolor       = &WHITE_YELLOW
    INT         hcolor       = &MAGENTA_WHITE
    INT         edit_color   = &CYAN_WHITE
    INT         phone_color  = &WHITE_YELLOW
    INT         scrn_color   = &BLUE_WHITE
    INT         hello_color  = &CYAN_WHITE
    INT         menu_color   = &WHITE_BLUE
    INT         cursor_color = &MAGENTA_WHITE
    LOGICAL     at_top       = .t.
    LOGICAL     at_bottom    = .f.
    LONG        minus_one    = -1
    UINT        MustPack     = 0
ENDDEF

*******************************************************************************
*
*  Bibliothek-Funktionen, die nicht in einer HDR-Daei enthalten sind
*
*******************************************************************************

FUNCTION INT _find_key_mn PROTOTYPE
 PARAMETERS VALUE INT, VALUE INT

*******************************************************************************
*
*  Globale Fehler werden durch diese Prozedur behandelt
*
*******************************************************************************

PROCEDURE error_proc
   CLEAR
   @ 10,10 ?? "Sie haben mich geschafft!!!"
   @ 11,10 ?? e_message()

ENDPRO

*******************************************************************************
*
*     Funktion: MOVE_CUR
* Beschreibung: Markierungsbalken in Tabellen-Anzeige bewegen
*
*
*******************************************************************************

PROCEDURE move_cur
 PARAMETERS VALUE INT target

   curcolor( cur_row, 15, fcolor, &CURLENGTH )
   curcolor( target, 15, hcolor, &CURLENGTH )
   cur_row = target

ENDPRO

*******************************************************************************
*
*     Funktion: GET_INPUT
* Beschreibung: Sichert einen Teil des Bildschirms, ffnet ein Fenster und
*               ermglcht Eingabe von Name und Telefonnummer. Die Routine
*               wird von ADD, EDIT und SEARCH benutzt.
*
*******************************************************************************

PROCEDURE get_input
 PARAMETERS CONST CHAR add_edit, CHAR(30) name, CHAR(30) phone_number, ;
            value logical EnterRecord

   VARDEF
      INT   old_colorh, old_colors
      INT   scrn
   ENDDEF

   old_colorh = __color_enhcd
   old_colors = __color_std

   scrn = savescrn( 10,16,16,64 )

   fill( 10,16,16,64, &SINGLE_BOX," ", edit_color, edit_color, 6 )
        __color_std   = edit_color
   __color_enhcd = &WHITE_BLACK

   @ 10,40-(len( add_edit )/2) ?? add_edit

   cursor_on

   @ 12,18 SAY "       Name: ";
           GET name
   if EnterRecord
     @ 13,18 SAY "       Tel.: ";
           GET phone_number
   endif

   READ

   cursor_off

   __color_enhcd = old_colorh
   __color_std   = old_colors

   restorescrn( scrn )

ENDPRO

*******************************************************************************
*
*     Funktion: EDIT_RECORD
* Beschreibung: Initialisiert einige Speichervariablen und ruft dann GET_INPUT
*               auf. Wenn lastkey() <> Esc ist, werden die Daten geschrieben
*
*******************************************************************************

PROCEDURE edit_record
 PARAMETERS CONST CHAR add_edit
   VARDEF
      CHAR(30) name, phone_number
   ENDDEF

   name = phone->name
   phone_number = phone->phone

   DO get_input WITH add_edit, name, phone_number, .t.

   IF lastkey() <> &K_ESC
        REPLACE phone->name  WITH name
        REPLACE phone->phone WITH phone_number
   ENDIF

ENDPRO

*******************************************************************************
*
*     Funktion: DISPLAY_REC
* Beschreibung: Zeigt Name und Telefonnummer in der aktuellen Zeile an. Wenn
*               ein Satz gelscht ist, wir ein "*" angezeigt.
*
*******************************************************************************

PROCEDURE display_rec
 PARAMETERS VALUE INT r
   VARDEF
      INT   old_color
   ENDDEF

   @ r ,15
   IF deleted()
      ?? "*"
   ELSE
      ?? " "
   ENDIF

        old_color   = __color_std
   __color_std = phone_color
   @ r,16
   ?? phone->name
   @ r,16+24
   ?? phone->phone

   __color_std = old_color

ENDPRO

*******************************************************************************
*
*     Funktion: NEWLINE
* Beschreibung: Fgt eine neue Zeile am Anfang oder Ende des Fensters ein.
*               Rollt den Fensterinhalt auf- oder abwrts, abhngig vom
*               Parameter 'direction'.
*
*******************************************************************************

PROCEDURE newline
 PARAMETERS VALUE INT direction

   curcolor( cur_row, 15, fcolor, &CURLENGTH )
   scroll( direction, top_row, top_col, bot_row, bot_col, 1, fcolor )
   DO display_rec WITH cur_row
   curcolor( cur_row, 15, hcolor, &CURLENGTH )

ENDPRO

*******************************************************************************
*
*     Funktion: REHIGHLIGHT
* Beschreibung: Zeigt den Auswahlcursor wieder invers an. Die Routine wird
*               nach dem Rollen der Anzeige oder nach SEARCH aufgerufen,
*               wenn ein Satz gefunden wurde.
*
*******************************************************************************

PROCEDURE rehighlight

   DO display_rec WITH cur_row
   curcolor( cur_row, 15, hcolor, &CURLENGTH )

ENDPRO

*******************************************************************************
*
*     Funktion: DISPLAY_NAMES
* Beschreibung: Gibt 12 Zeilen Information auf dem Bildschirm aus, beginnend
*               ab top_row
*
*******************************************************************************

PROCEDURE display_names
VARDEF
    INT     counter, row_pos
    LONG    current_record
ENDDEF

        counter = 12
        row_pos = 9

        current_record = recno()
        GO recno() - ( cur_row - top_row )

        DO WHILE .NOT. eof() .AND. COUNTER > 0

   DO display_rec WITH row_pos

      row_pos = row_pos + 1
      counter = counter - 1

      SKIP

   ENDDO

   GOTO current_record          && obersten angezeigten Satz "aktuell
                                && machen"

ENDPRO

*******************************************************************************
*
*     Funktion: IF_FOUND
* Beschreibung: Prft, ob ein Satz nach SEARCH/NEXT gefunden wurde. Wenn ja,
*               wird die Anzeige so positioniert, da cur_row gleich top_row
*               wird, d.h. der gefundene Satz wird in der oberen Tabellen-
*               zeile angezeigt.
*
*******************************************************************************

PROCEDURE if_found
 PARAMETERS VALUE ULONG old_rec

   IF found()
      cur_row = top_row
      @ top_row, 15 TO bot_row, 64 CLEAR
      DO display_names
      DO rehighlight
   ELSE
      GOTO old_rec
      sound( 150, 2 )
   ENDIF

ENDPRO

*******************************************************************************
*
*     Funktion: SEARCH_NAME
* Beschreibung: Sucht nach einem Namen
*
*******************************************************************************

PROCEDURE search_name
   VARDEF
      ULONG start_rec
   ENDDEF

   start_rec = recno()

   srch_name  = ""
   srch_phone = ""

   GO TOP

   DO get_input WITH " Suchen ", srch_name, srch_phone, .f.

   srch_name  = lower( ltrim( rtrim( srch_name )))

   IF (len(srch_name) > 0) .AND. lastkey() <> &K_ESC
     LOCATE FOR srch_name = substr(lower( phone->name ),1,len(srch_name))
   ENDIF

   IF lastkey() <> &K_ESC
      DO if_found WITH start_rec
   ENDIF

ENDPRO

*******************************************************************************
*
*     Funktion: TSR_PROC
* Beschreibung: Diese Funktion wird aufgerufen, wenn der Benutzer die Tasten-
*               Kombination <Ctrl><LeftShift> drckt.
*
*               Dies ist der Haupteinstieg fr das TSR-Programm.
*
*******************************************************************************

PROCEDURE tsr_proc
   VARDEF
        INT     s, menu_id, t, oldrow, oldcol, oldscolor, oldecolor
        UINT    k
        LONG    current_rec
        LOGICAL OldCursor
   ENDDEF

    *---Fehlerprozedur definieren

   ON ERROR DO error_proc

    *---Cursorposition sichern

   DO scrn_direct       && Hierdurch wird das Laufzeitsystem veranlat, die
                        && aktuelle Cursorposition des Systems ber BIOS
                        && zu lesen.

   oldrow = row()
   oldcol = col()

    *---Aktuelle Attribute sichern
        oldscolor = __color_std
        oldecolor = __color_enhcd

    *---Cursorzustand sichern

        OldCursor = __cursor
        cursor_off

    *---Bildschirm sichern

        do save_screen

    *---Teilbildschirm sichern und Startmeldung ausgeben

   s = savescrn( 6,21,19,61 )
   fill( 6,21,19,61,&SINGLE_BOX," ",hello_color,hello_color,6 )

    *---Meldung

   __color_std = hello_color

   @  9,30 ?? "    TSR-Programm"
   @ 11,30 ?? "         mit"
   @ 13,30 ?? "         FCO"
   @ 16,29 ?? "weiter mit Taste..."

    *---Datei ffnen

        USE phone_name ALIAS phone
        GOTO TOP
    *---Titel eine Weile anzeigen

   k = delay(2)         && 2 Sekunden verzgern
   restorescrn( s )     && alten Bildschirm-Inhalt zurckholen

    *---Rahmen anzeigen

   fill( 8,14,22,66,&SINGLE_BOX," ",phone_color, phone_color,0)
   __color_std = phone_color
   @ 8,16 SAY "Name"
   @ 8,40 SAY "Tel."

    *---Namen und Telefonnummern anzeigen

   cur_row = 9
   DO display_names
   curcolor( cur_row, top_col, hcolor, &CURLENGTH )

    *---Men aufbauen

   fill( 5,14,7,66,&SINGLE_BOX," ",menu_color, menu_color,0)

   __color_std = menu_color
   __color_enhcd = cursor_color

    *---das eigentliche Men

   @ 6,17 PROMPT "Suchen"       ID &M_SEARCH
   @ 6,26 PROMPT "Weitersuchen" ID &M_NEXT
   @ 6,41 PROMPT "Neu"          ID &M_ADD
   @ 6,47 PROMPT "ndern"       ID &M_EDIT
   @ 6,56 PROMPT "Lschen"      ID &M_DELETE

   menu_id = 0

   REPEAT

      MENU TO menu_id RETURN     && Rckkehr bei jedem Tastendruck auer bei
                                 && Cursortasten

      k = lastkey()
         t = _find_key_mn( k, menu_id )

         IF t <> -1
                  menu_id = t
                  k = 13
      ENDIF

           IF k = 13
         DO CASE
            CASE menu_id = &M_SEARCH
               DO search_name

            CASE menu_id = &M_NEXT
               IF len(srch_name) = 0
                  SOUND( 150, 2 )
               ELSE
                  current_rec = recno()
                  CONTINUE
                  DO if_found WITH current_rec
               ENDIF

            CASE menu_id = &M_ADD
               curcolor( cur_row, 15, fcolor, &CURLENGTH )
               APPEND BLANK
               DO edit_record WITH " Neu "

               IF reccount() > ( bot_row - top_row + 1 )
                  cur_row = bot_row
                  GOTO BOTTOM
                  DO display_names
                  move_cur( bot_row )
               ELSE
                  move_cur( top_row+recno() - 1 )
                  DO display_names
                  GOTO recno()
               ENDIF
               DO rehighlight

            CASE menu_id = &M_EDIT
               DO edit_record WITH " ndern "
               DO rehighlight

            CASE menu_id = &M_DELETE
               IF deleted()
                  RECALL
                  MustPack = MustPack - 1
               ELSE
                  DELETE
                  MustPack = MustPack + 1
               ENDIF
               DO rehighlight

         ENDCASE
      ELSE
         DO CASE
            CASE k = &K_UP
               IF recno() = 1
                  LOOP
               ENDIF

               IF cur_row = top_row
                  SKIP -1
                  DO newline WITH &SCRL_DOWN
               ELSE
                  SKIP -1
                  move_cur( cur_row - 1 )

               ENDIF

            CASE k = &K_DOWN
               IF recno() = reccount()
                  LOOP
               ENDIF

               IF cur_row = bot_row
                  SKIP
                  DO newline WITH &SCRL_UP
               ELSE
                  SKIP
                  move_cur( cur_row + 1 )

               ENDIF

         ENDCASE

      ENDIF

   UNTIL k = &K_ESC
   clear menu

   if MustPack > 0
     pack
   endif

   CLOSE phone
   do restore_area

  *---Cursor und Farben wiederherstellen
        __color_std   = oldscolor
        __color_enhcd = oldecolor

   if OldCursor
     cursor_on
   else
     cursor_off
   endif

   @ oldrow, oldcol

ENDPRO

*******************************************************************************
*
*     Funktion: Unloader
* Beschreibung: Diese Prozedur wird durch die dem Tastenkombination
*               <Ctrl><RightShift> aufgerufen und entfernt das TSR-Programm
*               aus dem Arbeitsspeicher.
*
*******************************************************************************
PROCEDURE unloader
  VARDEF
    INT oldrow, oldcol, savedscr, oldscolor, oldecolor
  ENDDEF

  DO scrn_direct

  oldrow = row()
  oldcol = col()

  oldscolor = __color_std
  oldecolor = __color_enhcd

  savedscr = savescrn( 9, 13, 12, 68 )

  __color_std = phone_color

  fill( 9,13,12,68,&DOUBLE_BOX," ",phone_color, phone_color, 6 )
  @10, 15 say "TSR aus dem Speicher entfernt - weiter mit Taste..."

  do get_key

  restorescrn( savedscr )

  __color_std   = oldscolor
  __color_enhcd = oldecolor

  @oldrow, oldcol

  *--- Dieser Aufruf mu als letzter in der Routine stehen
  UNLOAD_TSR()
ENDPRO


*******************************************************************************
*
*    Function: FCO_MAIN
* Description: Hauptprogramm
*
*******************************************************************************


PROCEDURE FCO_main
 PARAMETERS char cmd_line

 VARDEF
   CHAR temp_path
 ENDDEF

        set exact off

    *---prfen, ob Datenbank existiert

        temp_path  = trim( getenv( "PHONE" ) )
        IF len( temp_path ) = 0
          *--- Pfadbezeichnung bilden
          temp_path = chr( curdrive() + 'A' ) + ":" +  curdir( 0 ) + "\"
        ELSE
          IF right( temp_path, 1 ) <> "\"
            *--- Backslash hinzufgen
            temp_path = temp_path + "\"
          ENDIF
          IF left( temp_path, 1 ) <> "\" .AND. substr(temp_path, 2, 1) <> ":"
            *--- fhrender Backslash, wenn kein Lauferkbuchstabe vorhanden
            temp_path = "\" + temp_path
          ENDIF
        ENDIF

   phone_name = temp_path + "PHONENAM.DBF"
   IF .NOT. exist( phone_name )
      BUILD phone_name FROM ALIAS phone
   ENDIF

    *---keine CAPS/NUM-Anzeige

   SET SCOREBOARD OFF

    *---welche Farben benutzen wir?

   IF .NOT. iscolor() .OR. "mono" $ lower( cmd_line )
      STORE &WHITE_BLACK TO edit_color, phone_color, ;
         scrn_color, hello_color, menu_color, fcolor
      STORE &BLACK_WHITE TO cursor_color, hcolor
   ENDIF

    *---prfen, ob Programm bereits installiert ist
    IF .NOT. tsr_install(9132)                && eindeutiger Kenner
        ? "FEHLER:  TSR ist bereits installiert."
        ?
    ELSE                                      && noch nicht installiert
        IF proc_entry( tsr_proc, "<Ctrl><LeftShift>", &TSR_CALL_INT28 ) .AND.;
           proc_entry( unloader, "<Ctrl><RightShift>", &TSR_CALL_INT28 )

        *--- &TSR_CALL_INT28 wird fr den Timer Interrupt bentigt
        *    und auch zum Lschen aus dem Speicher, da UNLOAD_TSR
        *    DOS aufrufe benutzt.

            ? "TSR installiert:"
            ?
            ? "<Ctrl><LeftShift>  zum aktivieren,"
            ? "<Ctrl><RightShift> zum Entfernen aus dem Speicher."
            ?
            ? "Die Datenbank wurde wie folgt angelegt:"
            ? phone_name
            ?
            ? "Dies kann durch Setzen der Umgebungsvariablen PHONE"
            ? "gendert werden:"
            ?
            ? "z.B.:  SET PHONE=C:\PHONEMGR  in AUTOEXEC.BAT"
            ?
            tsr( 4096*4 )
        ENDIF
    ENDIF

    ? "FEHLER: kann das Programm nicht als TSR laden."

ENDPRO

*---EOF TSR.PRG
